// Author: Olivier Loisel (CREST). Date: June 2020.
// This Dynare file is called by the Matlab file ISP_Fig2_check.m, which checks Figure 2 and the Supplementary Appendix S.2 in the paper "The Implementation of Stabilization Policy" (O. Loisel, June 2020).
// It is consistent at least with version 4.3.0 of Dynare.
// It determines whether Blanchard and Kahn's (1980) root-counting and no-decoupling conditions are met under the rule (41).
// When there are met, it computes the impulse-reponse functions to the two exogenous shocks.
// The names of the parameters that appear also in the paper correspond to their notations in the paper (with the suffix "_ru" standing for "rule").
// The variables of type Xtilde (where X is pi, y, or i) correspond to E_t[X_{t+1}].

var pi, y, i, pitilde, ytilde, itilde, eta, u; 
varexo epseta, epsu; 
parameters beta_ru, sigma_ru, kappa_ru, lambda_ru, rho_eta_ru, rho_u_ru, pi1, pi2, pi3, y1, y2, y3, i0, i1, i2, i3;

load('Param.mat');
beta_ru = beta;
sigma_ru = sigma;
kappa_ru = kappa;
lambda_ru = lambda;
rho_eta_ru = rho_eta;
rho_u_ru = rho_u;

load('ISP_Fig2_check.mat');
pi1 = F_pi_ext(3);
pi2 = F_pi_ext(2);
pi3 = F_pi_ext(1);
y1 = F_y_ext(3);
y2 = F_y_ext(2);
y3 = F_y_ext(1);
i0 = G_ext(4);
i1 = G_ext(3);
i2 = G_ext(2);
i3 = G_ext(1);

model (linear);
pi = pitilde(-1)+epsu;
y = ytilde(-1)+epseta;
i = itilde(-1);
pitilde = beta_ru*pitilde(+1)+kappa_ru*ytilde+rho_u_ru*u;
ytilde = ytilde(+1)-1/sigma_ru*(itilde-pitilde(+1))+rho_eta_ru*eta;
i0*itilde+ i1*itilde(-1) + i2*itilde(-2) + i3*itilde(-3) = pi1*pitilde(-1) + pi2*pitilde(-2) + pi3*pitilde(-3) + y1*ytilde(-1) + y2*ytilde(-2) + y3*ytilde(-3)
                                                           + pi1*epsu + pi2*epsu(-1) + pi3*epsu(-2) + y1*epseta + y2*epseta(-1) + y3*epseta(-2) ;

// The itilde rule above is equivalent to the following i rule:
// i0*i+ i1*i(-1) + i2*i(-2) + i3*i(-3) = pi1*pi(-1) + pi2*pi(-2) + pi3*pi(-3) + y1*y(-1) + y2*y(-2) + y3*y(-3).
// However, Dynare fails to see this equivalence, because it fails to see the equivalence between "i0*itilde+... = pi1*pitilde(-1)+..." and "i0*itilde(-1)+... = pi1*pitilde(-2)+...".
// Therefore, Dynare wrongly identifies an infinite eigenvalue under the i rule. This is why I use the itilde rule instead of the i rule.

eta = rho_eta_ru*eta(-1) + epseta;
u = rho_u_ru*u(-1) + epsu;
end;

shocks;
var epseta = 1;
var epsu = 1;
corr epseta, epsu = 0;
end;

options_.noprint = 1;
check;

stoch_simul(irf=20,order=1,noprint,nograph);
error = info(1);